﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Common;

namespace HeuristicLab.Parameters {
  [StorableClass]
  public class ParameterNameBinding : IParameterBinding {
    [Storable]
    private IParameter source;
    public IParameter Source {
      get { return source; }
    }
    private ILookupParameter sourceAsLookup;

    [Storable]
    private ILookupParameter target;
    public ILookupParameter Target {
      get { return target; }
    }

    [StorableConstructor]
    protected ParameterNameBinding(bool deserializing) { }
    protected ParameterNameBinding(ParameterNameBinding original, Cloner cloner) {
      this.source = cloner.Clone(source);
      this.target = cloner.Clone(target);
      this.sourceAsLookup = this.source as ILookupParameter;
      RegisterEventHandlers();
    }
    public ParameterNameBinding(ILookupParameter target, IParameter source) {
      if (source == null) throw new ArgumentNullException("source");
      if (target == null) throw new ArgumentNullException("target");
      this.source = source;
      this.target = target;
      this.sourceAsLookup = this.source as ILookupParameter;
    }

    public object Clone() {
      return Clone(new Cloner());
    }

    public virtual IDeepCloneable Clone(Cloner cloner) {
      return new ParameterNameBinding(this, cloner);
    }

    public void Bind() {
      RegisterEventHandlers();
      ExecuteBinding();
    }

    public void Unbind() {
      DeregisterEventHandlers();
    }

    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      this.sourceAsLookup = this.source as ILookupParameter;
      RegisterEventHandlers();
    }

    private void RegisterEventHandlers() {
      if (sourceAsLookup != null)
        sourceAsLookup.ActualNameChanged += new EventHandler(source_NameChanged);
      else source.NameChanged += new EventHandler(source_NameChanged);
    }

    private void DeregisterEventHandlers() {
      if (sourceAsLookup != null)
        sourceAsLookup.ActualNameChanged -= new EventHandler(source_NameChanged);
      else source.NameChanged -= new EventHandler(source_NameChanged);
    }

    private void source_NameChanged(object sender, EventArgs e) {
      ExecuteBinding();
    }

    private void ExecuteBinding() {
      if (sourceAsLookup != null) {
        if (target.ActualName != sourceAsLookup.ActualName)
          target.ActualName = sourceAsLookup.ActualName;
      } else target.ActualName = source.Name;
    }
  }
}
