﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Common;

namespace HeuristicLab.Parameters {
  [StorableClass]
  public class ParameterValueBinding : IParameterBinding {
    [Storable]
    private IValueParameter target;
    public IValueParameter Target {
      get { return target; }
    }
    [Storable]
    private IValueParameter source;
    public IValueParameter Source {
      get { return source; }
    }

    [StorableConstructor]
    protected ParameterValueBinding(bool deserializing) { }
    protected ParameterValueBinding(ParameterValueBinding original, Cloner cloner) {
      this.source = cloner.Clone(source);
      this.target = cloner.Clone(target);
      RegisterEventHandlers();
    }
    public ParameterValueBinding(IValueParameter target, IValueParameter source) {
      if (target == null) throw new ArgumentNullException("destination");
      if (source == null) throw new ArgumentNullException("source");
      this.target = target;
      this.source = source;
    }

    public object Clone() {
      return Clone(new Cloner());
    }

    public virtual IDeepCloneable Clone(Cloner cloner) {
      return new ParameterValueBinding(this, cloner);
    }

    public void Bind() {
      RegisterEventHandlers();
      ExecuteBinding();
    }

    public void Unbind() {
      DeregisterEventHandlers();
    }

    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      RegisterEventHandlers();
    }

    private void RegisterEventHandlers() {
      source.ValueChanged += new EventHandler(source_ValueChanged);
    }

    private void DeregisterEventHandlers() {
      source.ValueChanged -= new EventHandler(source_ValueChanged);
    }

    private void source_ValueChanged(object sender, EventArgs e) {
      ExecuteBinding();
    }

    private void ExecuteBinding() {
      target.Value = (IItem)source.Value.Clone();
    }
  }
}
